function [cma,cm]=emaxseg(img,k)
//   Expectation Maximization image segmentation
//
//   Input:
//          img: Gray level image
//          k: Number of classes
//   Output:
//          cma: clasification matrix
//          cm: class mean vector  
//
// How to run
//   Example: [cma,cm]=ExMaxSeg(img,3);
//
//

// Converting to gray scale range
img=floor(mat2gray(double(img))*255)+1;
cpy=img;           
img=img(:);         

// initialization of parameters
mx=max(img);
p=ones(1,k)*1/k;
v=ones(1,k)*mx;
cm=(1:k)*mx/(k+1);

// Pixel probaility of the image

ppr=zeros(1,mx);
for i=1:length(img),
         ppr(img(i)) =ppr(img(i)) + 1;
end
ppr=convol(ppr,[1,2,3,4,5,4,3,2,1])';
ppr=ppr(5:$-4);
ppr=ppr/sum(ppr);

inp=(1:length(ppr))';

// Expectation Maximization 
while(%t)
        
        prob = distrib(cm,v,p,inp);
        scal = sum(prob,2)+%eps;
        logpr=sum(ppr.*log(scal));

        for i=1:k
                p1=ppr.*prob(:,i)./scal;
                p(i) = sum(p1);
                cm(i) = sum(inp.*p1)/p(i);
                v1 = (inp-cm(i)).^2;
                v(i)=sum(v1.*p1)/p(i);
        end
        p = p + 1e-3;
        p = p/sum(p);

        prob = distrib(cm,v,p,inp);
        scal = sum(prob,2)+%eps;
        nlogpr=sum(ppr.*log(scal));
        // condition for exiting the loop 
        if((nlogpr-logpr)<0.0001)
             break;
        end

end

// calculating classification matrix

[r,c]=size(cpy);
cma=zeros(r,c);

for i=1:r
for j=1:c
  zz=abs(cm-cpy(i,j));
  zz=find(zz==min(zz));
  cma(i,j)=cm(zz(1)); 
end
end

endfunction

function y=distrib(m,v,g,x)
 amp = g./sqrt(2*%pi*v);
 d=repmat(x,1,length(m))-repmat(m,length(x),1);
 y=repmat(amp,length(x),1).*exp(-0.5*(d.*d)./repmat(v,length(x),1));
endfunction

